@@ -12,12 +12,17 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <libintl.h>
+#include <locale.h>
 
 #include <gtk/gtk.h>
 #include <glib.h>
+#include <gdk/gdkkeysyms.h>
 
 #include <gpe/pixmaps.h>
 #include <gpe/event-db.h>
+#include <gpe/question.h>
+#include <gpe/schedule.h>
 
 #include "event-ui.h"
 #include "globals.h"
@@ -25,15 +30,145 @@
 
 #include "gtkdatesel.h"
 
+#define _(x) gettext(x)
+
 static GSList *strings;
 static GSList *day_events[24], *untimed_events;
-static GtkWidget *day_list;
+GtkWidget *day_list;
 static GtkWidget *datesel;
 static GtkAdjustment *scroll_adjustment;
 
 static GtkStyle *light_style, *dark_style, *time_style;
 static GdkColor light_color, dark_color, time_color;
 
+
+void
+delete_event (event_t ev)
+{
+  event_t ev_real;
+  event_details_t ev_d;
+  recur_t r;
+  
+  ev_real = (event_t)ev->cloned_ev;
+  ev_d = event_db_get_details (ev_real);
+     
+  if (ev_real->recur)
+    {
+      if (gpe_question_ask(
+        _("Delete all recurring entries?\n(If no, delete this instance only)"), 
+        _("Question"), "question", "!gtk-no", NULL, "!gtk-yes", NULL, NULL))
+        {			  
+          if (ev_real->flags & FLAG_ALARM)
+            {
+              schedule_cancel_alarm (ev_real->uid, ev->start);
+              schedule_next (0,0);
+            }
+          event_db_remove (ev_real);
+        }
+      else 
+        {
+          r = event_db_get_recurrence (ev_real);
+          r->exceptions = g_slist_append (r->exceptions, (void *)ev->start);
+          event_db_flush (ev_real);
+        }
+    }
+  else
+    {			  
+      if (ev_real->flags & FLAG_ALARM)
+        {
+          schedule_cancel_alarm (ev_real->uid, ev->start);
+          schedule_next (0,0);
+        }
+      event_db_remove (ev_real);
+  }
+  
+  update_all_views ();
+  event_db_forget_details (ev_real);
+}
+
+
+static gboolean
+list_key_press_event (GtkWidget *clist, GdkEventKey *k, gpointer user_data)
+{
+  int row;
+  event_t ev;
+  GtkWidget *w;
+  int lastrow = (int)g_object_get_data(G_OBJECT(clist),"last-row");
+  
+  row = (int)g_object_get_data(G_OBJECT(clist),"selected-row");
+  
+  if (k->keyval == GDK_Up)
+  {
+    gtk_clist_select_row(GTK_CLIST(clist),row-1, 0);
+    if (!row)
+      {
+        gtk_widget_child_focus(gtk_widget_get_toplevel(GTK_WIDGET(clist)),
+		                         GTK_DIR_UP);
+        return TRUE;
+      }
+    else
+      return FALSE;
+  }
+  
+  if (k->keyval == GDK_Down)
+  {
+    gtk_clist_select_row(GTK_CLIST(clist),row + 1, 0);
+    if (row == lastrow)
+      {
+        gtk_widget_child_focus(gtk_widget_get_toplevel(GTK_WIDGET(clist)),
+		                      GTK_DIR_TAB_FORWARD);
+        lastrow = -1;
+        return TRUE;
+      }
+    else
+      {
+        lastrow = row;
+        return FALSE;
+      }
+  }
+  
+  if (k->keyval == GDK_Return)
+  {
+    if (row >= 0)
+      {
+        struct tm tm;
+        ev = gtk_clist_get_row_data(GTK_CLIST (clist), row);
+        if (ev)
+          {
+            w = edit_event(ev);
+          }
+        else
+          {
+            char *t;
+            localtime_r (&viewtime, &tm);
+            if (gtk_clist_get_text (GTK_CLIST (clist), row, 0, &t))
+              strptime (t, TIMEFMT, &tm);
+            w = new_event (mktime (&tm), 1);
+          }
+        gtk_widget_show (w);
+      }
+    return TRUE;
+  }
+ 
+  if (k->keyval == GDK_Delete)
+  {
+    row = (int)g_object_get_data(G_OBJECT(clist),"selected-row");
+    if (row >= 0)
+      {
+        ev = gtk_clist_get_row_data(GTK_CLIST (clist), row);
+        if ((ev) && (gpe_question_ask (_("Delete event?"), _("Question"), "question",
+			   "!gtk-no", NULL, "!gtk-yes", NULL, NULL)))
+          {
+            delete_event(ev);
+            g_object_set_data(G_OBJECT(clist),"selected-row",(void *)-1);
+          }
+      }
+    return TRUE;
+  }
+  
+  return FALSE;
+}
+
 static char *
 format_event (event_t ev)
 {
@@ -86,8 +221,15 @@ selection_made (GtkWidget      *clist,
 {
   event_t ev;
   GtkWidget *w;
-    
-  if (event->type == GDK_2BUTTON_PRESS)
+  
+  g_object_set_data(G_OBJECT(clist),"last-row",
+    g_object_get_data(G_OBJECT(clist),"selected-row"));
+  g_object_set_data(G_OBJECT(clist),"selected-row",(void *)row);
+
+  if (!event) return;
+  
+  if ((event->type == GDK_2BUTTON_PRESS) 
+     || ((event->type == GDK_BUTTON_RELEASE) && (column == 1)))
     {
       struct tm tm;
       
@@ -134,7 +276,8 @@ day_view_update ()
   PangoLayout *pl = gtk_widget_create_pango_layout (GTK_WIDGET (day_list), NULL);
   PangoRectangle pr;
   double pos;
-     
+  gchar *last_text = NULL;
+  
   widget_width = day_list->allocation.width;
 
   if (! light_style)
@@ -157,12 +300,12 @@ day_view_update ()
       time_style = gtk_style_copy (gtk_widget_get_style (day_list));
       
       for (j = 0; j < 5; j++) 
-	{
-	  light_style->base[j] = light_color;
-	  dark_style->base[j] = dark_color;
-	  time_style->base[j] = time_color;
-	  time_style->fg[j] = light_color;
-	}
+        {
+          light_style->base[j] = light_color;
+          dark_style->base[j] = dark_color;
+          time_style->base[j] = time_color;
+          time_style->fg[j] = light_color;
+        }
     }
   
   gtk_clist_freeze (GTK_CLIST (day_list));
@@ -171,7 +314,7 @@ day_view_update ()
   if (strings)
     {
       for (iter = strings; iter; iter = iter->next)
-	g_free (iter->data);
+        g_free (iter->data);
       g_slist_free (strings);
       strings = NULL;
     }
@@ -220,11 +363,12 @@ day_view_update ()
       tm_end.tm_sec = 0;
       end = mktime (&tm_end);
 
-      if (day_events[hour]) event_db_list_destroy (day_events[hour]);
+      if (day_events[hour]) 
+        event_db_list_destroy (day_events[hour]);
       day_events[hour] = event_db_untimed_list_for_period (start, end-1, FALSE);
       
       for (iter = day_events[hour]; iter; iter = iter->next)
-	((event_t)iter->data)->mark = FALSE;
+	    ((event_t)iter->data)->mark = FALSE;
     }
 
   for (hour = 0; hour <= 23; hour++)
@@ -240,27 +384,33 @@ day_view_update ()
       strftime (buf, sizeof (buf), "%R", &tm_start);
 
       for (iter = day_events[hour]; iter; iter = iter->next)
-	{
-	  ev = (event_t) iter->data;
+        {
+          ev = (event_t) iter->data;
+    
+          if (ev->mark == FALSE)
+            {
+              struct tm tm;
+              ev->mark = TRUE;
+              text = format_event (ev);
+              g_slist_append (strings, text);
+    
+              if (day_view_combined_times)
+                {
+                  localtime_r (&ev->start, &tm);
+                  strftime (buf, sizeof (buf), "%R", &tm);
+                }
+              break;
+            }
+        } 
 
-	  if (ev->mark == FALSE)
-	    {
-	      struct tm tm;
-	      ev->mark = TRUE;
-	      text = format_event (ev);
-	      g_slist_append (strings, text);
-
-	      if (day_view_combined_times)
-		{
-		  localtime_r (&ev->start, &tm);
-		  strftime (buf, sizeof (buf), "%R", &tm);
-		}
-	      break;
-	    }
-	} 
-
-      line_info[1] = text;
+      if (!last_text || !text || (strcmp(last_text,text)))
+        line_info[1] = text;
+      else 
+        line_info[1] = NULL;
       line_info[0] = buf;
+      if (last_text) 
+        g_free(last_text);
+      last_text = g_strdup(text);
 
       pango_layout_set_text (pl, buf, strlen (buf));
       pango_layout_get_pixel_extents (pl, &pr, NULL);
@@ -268,42 +418,50 @@ day_view_update ()
 
       if (w > width) width = w;
 	
-      gtk_clist_append (GTK_CLIST (day_list), line_info);
-      gtk_clist_set_row_data (GTK_CLIST (day_list), row, ev);
-
-      if (ev) 
-	{
-	  GdkPixmap *pmap;
-	  GdkBitmap *bmap;
-	  
-    	  if ((ev->flags & FLAG_ALARM) && ev->recur)
-	    {
-	      if (gpe_find_icon_pixmap ("bell_recur", &pmap, &bmap))
-		gtk_clist_set_pixtext (GTK_CLIST (day_list), row, 1, text, 5,
-				       pmap, bmap);
-	    }
-	  else if (ev->flags & FLAG_ALARM)
-	    { 
-	      if (gpe_find_icon_pixmap ("bell", &pmap, &bmap))
-		gtk_clist_set_pixtext (GTK_CLIST (day_list), row, 1, text, 5,
-				       pmap, bmap);
-	    }
-	  else if (ev->recur)
-	    {
-	      if (gpe_find_icon_pixmap ("recur", &pmap, &bmap))
-		gtk_clist_set_pixtext (GTK_CLIST (day_list), row, 1, text, 5,
-				       pmap, bmap);
-	    }
-	}
+      /* don't display very early/late slots if they are not used */
+      if (ev || ((hour > 5) && (hour < 23)))
+      {        
+        gtk_clist_append (GTK_CLIST (day_list), line_info);
+        gtk_clist_set_row_data (GTK_CLIST (day_list), row, ev);
+      }
       
-      gtk_clist_set_cell_style (GTK_CLIST (day_list), row, 1, 
-				day_events[hour] ? dark_style : light_style); 
-      row++;
+      if (ev)
+        {
+          GdkPixmap *pmap;
+          GdkBitmap *bmap;
+          
+          if ((ev->flags & FLAG_ALARM) && ev->recur)
+            {
+              if (gpe_find_icon_pixmap ("bell_recur", &pmap, &bmap))
+                gtk_clist_set_pixtext (GTK_CLIST (day_list), row, 1, text, 5,
+                           pmap, bmap);
+            }
+          else if (ev->flags & FLAG_ALARM)
+            { 
+              if (gpe_find_icon_pixmap ("bell", &pmap, &bmap))
+                gtk_clist_set_pixtext (GTK_CLIST (day_list), row, 1, text, 5,
+                           pmap, bmap);
+            }
+          else if (ev->recur)
+            {
+              if (gpe_find_icon_pixmap ("recur", &pmap, &bmap))
+                gtk_clist_set_pixtext (GTK_CLIST (day_list), row, 1, text, 5,
+                           pmap, bmap);
+            }
+        }
       
-      for (; iter; iter = iter->next)
+      /* don't display very early/late slots if they are not used */
+      if (ev || ((hour > 5) && (hour < 23)))
+        {
+          gtk_clist_set_cell_style (GTK_CLIST (day_list), row, 1, 
+                  day_events[hour] ? dark_style : light_style); 
+          row++;
+        }
+              
+  for (; iter; iter = iter->next)
 	{
 	  ev = (event_t) iter->data;
-
+      
 	  if (ev->mark)
 	    continue;
 
@@ -321,15 +479,15 @@ day_view_update ()
 	      line_info[0] = NULL;
 	    }
 
-	  line_info[1] = format_event (ev);
-	  g_slist_append (strings, line_info[1]);
-
-          gtk_clist_append (GTK_CLIST (day_list), line_info);
-	  gtk_clist_set_row_data (GTK_CLIST (day_list), row, ev);
-	  gtk_clist_set_cell_style (GTK_CLIST (day_list), row, 1, dark_style);
-	  row++;
-       } 
-    }
+        line_info[1] = format_event (ev);
+        g_slist_append (strings, line_info[1]);
+  
+        gtk_clist_append (GTK_CLIST (day_list), line_info);
+        gtk_clist_set_row_data (GTK_CLIST (day_list), row, ev);
+        gtk_clist_set_cell_style (GTK_CLIST (day_list), row, 1, dark_style);
+        row++;
+      } 
+  }
 
   for (i = 0; i < row; i++)
     gtk_clist_set_cell_style (GTK_CLIST (day_list), i, 0, time_style);
@@ -361,6 +519,29 @@ update_hook_callback()
   day_view_update ();
 }
 
+void
+day_free_lists()
+{
+  int hour;
+  if (untimed_events) 
+    event_db_list_destroy (untimed_events);
+  untimed_events = NULL;
+  for (hour = 0; hour <= 23; hour++)
+    if (day_events[hour]) 
+      {
+        event_db_list_destroy (day_events[hour]);
+        day_events[hour] = NULL;
+      }
+}
+
+
+gboolean
+list_exposed(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
+{
+  gtk_container_propagate_expose  (widget, day_list, event);
+  return FALSE;
+}
+
 GtkWidget *
 day_view(void)
 {
@@ -373,6 +554,7 @@ day_view(void)
   gtk_widget_show (datesel);
   
   day_list = gtk_clist_new (2);
+  gtk_clist_set_selection_mode(GTK_CLIST(day_list),GTK_SELECTION_SINGLE);
   gtk_widget_show (day_list);
  
   g_signal_connect (G_OBJECT (day_list), "select_row",
@@ -392,7 +574,16 @@ day_view(void)
   g_object_set_data (G_OBJECT (vbox), "update_hook",
 		     (gpointer) update_hook_callback);
 
-  scroll_adjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window));
+  scroll_adjustment = 
+    gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window));
+
+  g_signal_connect (G_OBJECT (day_list), "key_press_event", 
+		    G_CALLBACK (list_key_press_event), NULL);
+  g_signal_connect (G_OBJECT (scrolled_window), "expose-event",
+            G_CALLBACK (list_exposed), NULL);
+  g_object_set_data(G_OBJECT(day_list),"selected-row",(void *)0);
+
+  g_object_set_data(G_OBJECT(main_window),"datesel-day",datesel);
 
   return vbox;
 }
