@@ -18,6 +18,7 @@
 #include <langinfo.h>
 
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 
 #include <gpe/question.h>
 #include <gpe/pixmaps.h>
@@ -255,30 +256,32 @@ click_delete (GtkWidget *widget, GtkWidget *d)
      
   if (ev_real->recur)
     {
-      if (gpe_question_ask_yn (_("Delete all recurring entries?\n(If no, delete this instance only)")))
-	{			  
-	  if (ev_real->flags & FLAG_ALARM)
-	    {
-	      unschedule_alarm (ev);
-	      schedule_next (0,0);
-	    }
-	  
-	  event_db_remove (ev_real);
-	}
+      if (gpe_question_ask(
+        _("Delete all recurring entries?\n(If no, delete this instance only)"), 
+        _("Question"), "question", "!gtk-no", NULL, "!gtk-yes", NULL, NULL))
+        {			  
+          if (ev_real->flags & FLAG_ALARM)
+            {
+              unschedule_alarm (ev);
+              schedule_next (0,0);
+            }
+          
+          event_db_remove (ev_real);
+        }
       else 
-	{
-	  r = event_db_get_recurrence (ev_real);
-	  r->exceptions = g_slist_append (r->exceptions, (void *)ev->start);
-	  event_db_flush (ev_real);
-	}
+        {
+          r = event_db_get_recurrence (ev_real);
+          r->exceptions = g_slist_append (r->exceptions, (void *)ev->start);
+          event_db_flush (ev_real);
+        }
     }
   else
     {			  
       if (ev_real->flags & FLAG_ALARM)
-	{
-	  unschedule_alarm (ev);
-	  schedule_next (0,0);
-	}
+        {
+          unschedule_alarm (ev);
+          schedule_next (0,0);
+        }
       
       event_db_remove (ev_real);
   }
@@ -586,6 +589,80 @@ click_categories (GtkWidget *b, GtkWidget *w)
   ui = gpe_pim_categories_dialog (s->categories, G_CALLBACK (update_categories), s);
 }
 
+gboolean
+on_description_focus_in(GtkWidget* widget, GdkEventFocus *event,
+                        gpointer user_data)
+{
+  GtkTextIter iter;
+  GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
+  GObject *window = user_data;
+  
+  g_object_set_data(window,"multiline",(void*)TRUE);
+  gtk_text_buffer_get_start_iter(buf,&iter);
+  gtk_text_buffer_place_cursor(buf,&iter);
+  
+  return FALSE;
+}
+
+gboolean
+on_description_focus_out(GtkWidget* widget, GdkEventFocus *event,
+                        gpointer user_data)
+{
+  GObject *window = user_data;
+  
+  g_object_set_data(window,"multiline",(void*)FALSE);
+  return FALSE;
+}
+
+void        
+tv_move_cursor (GtkTextView *textview,
+                GtkMovementStep arg1,
+                gint arg2,
+                gboolean arg3,
+                gpointer user_data)
+{
+  GtkTextBuffer *buf = gtk_text_view_get_buffer(textview);
+  
+  int cnt = (int)g_object_get_data(G_OBJECT(textview),"cnt");
+  
+  if (arg1 == GTK_MOVEMENT_DISPLAY_LINES)
+    {
+      cnt += arg2;
+      if (cnt >= gtk_text_buffer_get_line_count(buf))
+        {
+          cnt = 0;
+          gtk_widget_child_focus(gtk_widget_get_toplevel(GTK_WIDGET(textview)),
+		                         GTK_DIR_DOWN);
+        }
+      else if (cnt < 0)
+        {
+          cnt = 0;
+          gtk_widget_child_focus(gtk_widget_get_toplevel(GTK_WIDGET(textview)),
+		                         GTK_DIR_UP);
+        }  
+      g_object_set_data(G_OBJECT(textview),"cnt",(void*)cnt);  
+    }
+}
+
+static gboolean
+event_ui_key_press_event (GtkWidget *widget, GdkEventKey *k, gpointer data)
+{
+  if (k->keyval == GDK_Escape)
+    {
+      edit_finished(widget);
+      return TRUE;
+    }
+  if (k->keyval == GDK_Return)
+    {
+      if (!g_object_get_data(G_OBJECT(widget),"multiline"))
+        {
+          click_ok(NULL,widget);
+          return TRUE;
+       }
+    }
+  return FALSE;
+}
+
 static GtkWidget *
 build_edit_event_window (void)
 {
@@ -733,11 +810,12 @@ build_edit_event_window (void)
 
   gtk_box_pack_start (GTK_BOX (menutypehbox), menutypelabel, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (menutypehbox), s->optionmenutype, TRUE, TRUE, 0);
-
   /* Building the summary wigets */
   summaryhbox         = gtk_hbox_new (FALSE, boxspacing);
   summaryentry        = gtk_entry_new ();
-  summarylabel        = gtk_label_new (_("Summary:"));
+  summarylabel        = gtk_label_new (_("Text:"));
+  
+  gtk_misc_set_alignment(GTK_MISC(summarylabel),0,0.5);
 
   gtk_box_pack_start (GTK_BOX (summaryhbox), summarylabel, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (summaryhbox), summaryentry, TRUE, TRUE, 0);
@@ -752,11 +830,16 @@ build_edit_event_window (void)
 
   gtk_combo_set_popdown_strings (GTK_COMBO (starttime), times);
   gtk_combo_set_popdown_strings (GTK_COMBO (endtime), times);
+  gtk_combo_set_use_arrows (GTK_COMBO(starttime), FALSE);
+  gtk_combo_set_use_arrows (GTK_COMBO(endtime), FALSE);
 
-  startdatelabel      = gtk_label_new (_("Start:"));
-  enddatelabel        = gtk_label_new (_("End:"));
-  starttimelabel      = gtk_label_new (_("at:"));
-  endtimelabel        = gtk_label_new (_("at:"));
+  startdatelabel      = gtk_label_new (_("Start at:"));
+  enddatelabel        = gtk_label_new (_("End at:"));
+  starttimelabel      = gtk_label_new (_("on:"));
+  endtimelabel        = gtk_label_new (_("on:"));
+  
+  gtk_misc_set_alignment(GTK_MISC(startdatelabel),0,0.5);
+  gtk_misc_set_alignment(GTK_MISC(enddatelabel),0,0.5);
 
   s->startdate        = gtk_date_combo_new ();
   s->enddate          = gtk_date_combo_new ();
@@ -770,27 +853,27 @@ build_edit_event_window (void)
 
   gtk_table_attach (GTK_TABLE (startendtable), startdatelabel, 0, 1, 0, 1,
                     0, 0, 0, boxspacing);
-  gtk_table_attach (GTK_TABLE (startendtable), s->startdate, 1, 2, 0, 1,
-                    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (startendtable), starttime, 1, 2, 0, 1,
+                    GTK_FILL, GTK_FILL, 0, 0);
   gtk_table_attach (GTK_TABLE (startendtable), starttimelabel, 2, 3, 0, 1,
                     0, 0, 0, boxspacing);
-  gtk_table_attach (GTK_TABLE (startendtable), starttime, 3, 4, 0, 1,
-                    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (startendtable), s->startdate, 3, 4, 0, 1,
+                    GTK_FILL, GTK_FILL, 0, 0);
   gtk_table_attach (GTK_TABLE (startendtable), enddatelabel, 0, 1, 1, 2,
                     0, 0, 0, boxspacing);
-  gtk_table_attach (GTK_TABLE (startendtable), s->enddate, 1, 2, 1, 2,
-                    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (startendtable), endtime, 1, 2, 1, 2,
+                    GTK_FILL, GTK_FILL, 0, 0);
   gtk_table_attach (GTK_TABLE (startendtable), endtimelabel, 2, 3, 1, 2,
                     0, 0, 0, boxspacing);
-  gtk_table_attach (GTK_TABLE (startendtable), endtime, 3, 4, 1, 2,
-                    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (startendtable), s->enddate, 3, 4, 1, 2,
+                    GTK_FILL, GTK_FILL, 0, 0);
 
   gtk_box_pack_start (GTK_BOX (vboxappointment), startendtable, FALSE, FALSE, 0);
 
   datetimetable       = gtk_table_new (2, 2, FALSE);
 
   datelabel           = gtk_label_new (_("Date:"));
-  gtk_misc_set_alignment (GTK_MISC (datelabel), 0.0, 0.0);
+  gtk_misc_set_alignment (GTK_MISC (datelabel), 0.0, 0.5);
 
   s->remindertimebutton = gtk_check_button_new_with_label (_("Time:"));
   g_signal_connect (G_OBJECT (s->remindertimebutton), "clicked",
@@ -798,6 +881,7 @@ build_edit_event_window (void)
 
   s->remindertime     = gtk_combo_new ();
   gtk_combo_set_popdown_strings (GTK_COMBO (s->remindertime), times);
+  gtk_combo_set_use_arrows (GTK_COMBO(s->remindertime), FALSE);
 
   gtk_table_attach (GTK_TABLE (datetimetable), datelabel, 0, 1, 0, 1,
                     0, GTK_EXPAND | GTK_FILL, 0, boxspacing);
@@ -826,7 +910,7 @@ build_edit_event_window (void)
 
   /* Categories */
   cathbox = gtk_hbox_new (FALSE, 0);
-  catbutton = gtk_button_new_with_label (_("Categories:"));
+  catbutton = gtk_button_new_with_label (_("Categories"));
   catlabel = gtk_label_new (NULL);
 
   gtk_box_pack_start (GTK_BOX (cathbox), catbutton, FALSE, FALSE, 0);
@@ -843,11 +927,18 @@ build_edit_event_window (void)
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (description), GTK_WRAP_WORD);
   gtk_text_view_set_editable (GTK_TEXT_VIEW (description), TRUE);
 
+  g_signal_connect(G_OBJECT(description), "focus-in-event", 
+                   G_CALLBACK(on_description_focus_in),window);
+  g_signal_connect(G_OBJECT(description), "focus-out-event", 
+                   G_CALLBACK(on_description_focus_out),window);
+  g_signal_connect(G_OBJECT(description),"move_cursor",
+                   G_CALLBACK(tv_move_cursor),NULL);
+
   descriptionlabel    = gtk_label_new (_("Description:"));
-  gtk_misc_set_alignment(GTK_MISC (descriptionlabel), 0.015, 0.0);
+  gtk_misc_set_alignment(GTK_MISC (descriptionlabel), 0, 0.5);
   gtk_widget_set_size_request (GTK_WIDGET (description), -1, 88);
   gtk_container_add (GTK_CONTAINER (descriptionhbox), description);
-
+  
   /* Putting the tab together */
   s->notebooktype = gtk_notebook_new ();
   gtk_notebook_set_show_tabs (GTK_NOTEBOOK (s->notebooktype), FALSE);
@@ -906,11 +997,15 @@ build_edit_event_window (void)
   gtk_container_set_border_width (GTK_CONTAINER (vboxrecur), border);
 
   /* Button box */
-  buttonbox           = gtk_hbox_new (FALSE, 0);
+  buttonbox           = gtk_hbutton_box_new ();
   buttonok            = gtk_button_new_from_stock (GTK_STOCK_SAVE);
   buttoncancel        = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
   buttondelete        = gtk_button_new_from_stock (GTK_STOCK_DELETE);
 
+  gtk_container_set_border_width (GTK_CONTAINER (buttonbox), border);
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(buttonbox),GTK_BUTTONBOX_END);
+  gtk_box_set_spacing(GTK_BOX(buttonbox),4);
+
   gtk_box_pack_start (GTK_BOX (buttonbox), buttondelete, TRUE, FALSE, 4);
   gtk_box_pack_start (GTK_BOX (buttonbox), buttoncancel, TRUE, FALSE, 4);
   gtk_box_pack_start (GTK_BOX (buttonbox), buttonok, TRUE, FALSE, 4);
@@ -1237,10 +1332,23 @@ build_edit_event_window (void)
   gpe_set_window_icon (window, "icon");
 
   gtk_window_set_default_size (GTK_WINDOW (window), 240, 320);
-
+  gtk_window_set_transient_for (GTK_WINDOW(window), GTK_WINDOW(main_window));
+  
+  /* if screen is large enough, make it a real dialog */
+  if (gdk_screen_width() > 320)
+    gtk_window_set_type_hint(GTK_WINDOW(window),GDK_WINDOW_TYPE_HINT_DIALOG);
+  
   g_signal_connect (G_OBJECT (window), "delete_event",
                     G_CALLBACK (edit_finished), NULL);
 
+  g_signal_connect (G_OBJECT (window), "key_press_event", 
+		            G_CALLBACK (event_ui_key_press_event), NULL);
+  gtk_widget_add_events (GTK_WIDGET (window), 
+                         GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
+  
+  GTK_WIDGET_SET_FLAGS (summaryentry, GTK_CAN_DEFAULT);
+  gtk_widget_grab_default(summaryentry);
+  
   return window;
 }
 
