@@ -15,6 +15,7 @@
 #include <langinfo.h>
 
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <glib.h>
 
 #include <gpe/event-db.h>
@@ -29,12 +30,15 @@ static GSList *day_events[32];
 static GtkWidget *datesel, *draw;
 static guint xp, xs, ys;
 static struct render_ctl *c_old;
+static guint active_day = 0;
 
 struct render_ctl
 {
   struct day_popup popup;
   gboolean valid;
   gboolean today;
+  gboolean active;
+  gboolean initialized;
 };
 
 #define TOTAL_DAYS (6 * 7)
@@ -62,7 +66,7 @@ button_press (GtkWidget *widget,
   y /= ys;
   y -= 1;
 
-  c = &rc[x + (y * 7)];
+  c = &rc[x + (y * 7) - week_offset];
   if (c->valid)
     {
       if (event->type == GDK_BUTTON_PRESS)
@@ -72,7 +76,7 @@ button_press (GtkWidget *widget,
 
 	  if (c != c_old) 
 	    {
-	      pop_window = day_popup (main_window, &c->popup);
+	      pop_window = day_popup (main_window, &c->popup, TRUE);
 	      c_old = c;
 	    }
 	  else 
@@ -83,18 +87,18 @@ button_press (GtkWidget *widget,
 	}
       
       else if (event->type == GDK_2BUTTON_PRESS)
-	{
-	  struct tm tm;
-	  time_t selected_time;
-	  localtime_r (&viewtime, &tm);
-	  tm.tm_year = c->popup.year - 1900;
-	  tm.tm_mon = c->popup.month;
-	  tm.tm_mday = c->popup.day;
-	  selected_time = mktime (&tm);
-	  if (pop_window) 
-	    gtk_widget_destroy (pop_window);
-	  set_time_and_day_view (selected_time);    
-	}
+        {
+          struct tm tm;
+          time_t selected_time;
+          localtime_r (&viewtime, &tm);
+          tm.tm_year = c->popup.year;
+          tm.tm_mon = c->popup.month;
+          tm.tm_mday = c->popup.day;
+          selected_time = mktime (&tm);
+          if (pop_window) 
+            gtk_widget_destroy (pop_window);
+          set_time_and_day_view (selected_time);    
+        }
     }
   
   return TRUE;
@@ -123,7 +127,28 @@ draw_expose_event (GtkWidget *widget,
   guint i, j;
   PangoLayout *pl;
   PangoRectangle pr;
-
+  static nl_item days[7];
+  
+  if (week_starts_monday) 
+  {
+  	  days[0] = ABDAY_2;
+  	  days[1] = ABDAY_3;
+  	  days[2] = ABDAY_4;
+  	  days[3] = ABDAY_5;
+  	  days[4] = ABDAY_6;
+  	  days[5] = ABDAY_7;
+  	  days[6] = ABDAY_1;
+  }
+  else {
+  	  days[0] = ABDAY_1;
+  	  days[1] = ABDAY_2;
+  	  days[2] = ABDAY_3;
+  	  days[3] = ABDAY_4;
+  	  days[4] = ABDAY_5;
+  	  days[5] = ABDAY_6;
+  	  days[6] = ABDAY_7;
+  }
+  
   g_return_val_if_fail (widget != NULL, TRUE);
   g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), TRUE);
 
@@ -191,14 +216,12 @@ draw_expose_event (GtkWidget *widget,
     {
       guint w;
       guint x = xp + (i * xs);
-      static const nl_item days[] = { ABDAY_2, ABDAY_3, ABDAY_4, ABDAY_5,
-				      ABDAY_6, ABDAY_7, ABDAY_1 };
       gchar *s = g_locale_to_utf8 (nl_langinfo (days[i]), -1,
                                    NULL, NULL, NULL);
       pango_layout_set_text (pl, s, strlen (s));
       pango_layout_get_pixel_extents (pl, &pr, NULL);
       w = pr.width;
-
+      
       gtk_paint_layout (widget->style,
 			widget->window,
 			GTK_WIDGET_STATE (widget),
@@ -212,7 +235,7 @@ draw_expose_event (GtkWidget *widget,
       for (j = 0; j < (TOTAL_DAYS / 7); j++)
 	{
 	  gchar *buffer;
-	  guint d = i + (7 * j);
+	  guint d = i + (7 * j) - week_offset;
 	  struct render_ctl *c = &rc[d];
 	  guint y = (j + 1) * ys;
 
@@ -263,11 +286,11 @@ draw_expose_event (GtkWidget *widget,
       guint x = xp + (i * xs);
       for (j = 0; j < (TOTAL_DAYS / 7); j++)
 	{
-	  guint d = i + (7 * j);
+	  guint d = i + (7 * j) - week_offset;
 	  struct render_ctl *c = &rc[d];
 	  guint y = (j + 1) * ys;
 
-	  if (c->today)
+	  if ((!force_today && c->active) || (force_today && c->today))
 	    {
 	      gdk_draw_rectangle (drawable, red_gc, FALSE,
 				  x, y, xs, ys);
@@ -320,7 +343,7 @@ month_view_update ()
   struct tm today;
   time_t t;
 
-  time (&t);
+  t = viewtime;
   localtime_r (&t, &today);
 
   localtime_r (&viewtime, &tm_start);
@@ -328,13 +351,16 @@ month_view_update ()
   month = tm_start.tm_mon;
 
   days = days_in_month (year, month);
+  wday = 1 - day_of_week(year, month + 1, 1);
 
   for (day = 0; day < TOTAL_DAYS; day++)
     {
+      gint rday = day + wday;
       struct render_ctl *c = &rc[day];
       if (c->popup.events)
-	c->popup.events = NULL;
-
+        c->popup.events = NULL;
+      if (rday == tm_start.tm_mday) active_day = day;
+      c->active = (day == active_day) ? TRUE : FALSE;
     }
 
   for (day = 1; day <= days; day++)
@@ -352,7 +378,7 @@ month_view_update ()
       tm_end.tm_min = 59;
       tm_end.tm_sec = 59;
       end = mktime (&tm_end);
-
+      
       if (!tm_start.tm_isdst) {
 	      start+=60*60;
 	      end+=60*60;
@@ -362,30 +388,36 @@ month_view_update ()
       day_events[day] = event_db_list_for_period (start, end);
 
       for (iter = day_events[day]; iter; iter = iter->next)
-	((event_t)iter->data)->mark = FALSE;
+        ((event_t)iter->data)->mark = FALSE;
     }
 
-  wday = 1 - day_of_week(year, month + 1, 1);
   for (day = 0; day < TOTAL_DAYS; day++)
     {
       gint rday = day + wday;
       struct render_ctl *c = &rc[day];
       c->popup.day = rday;
-      c->popup.year = year;
+      c->popup.year = year - 1900;
       c->popup.month = month;
       if (rday <= 0 || rday > days)
-	{
-	  c->valid = FALSE;
-	}
+        {
+          c->valid = FALSE;
+        }
       else
-	{
-	  c->valid = TRUE;
-	  c->popup.events = day_events[rday];
-	}
+        {
+          c->valid = TRUE;
+          c->popup.events = day_events[rday];
+        }
 
-      c->today = ((year == today.tm_year + 1900
+         c->today = ((year == today.tm_year + 1900
 		   && month == today.tm_mon
 		   && rday == today.tm_mday)) ? TRUE : FALSE;
+         if (!c->initialized)
+            {
+              c->active = c->today;
+              c->initialized = TRUE;
+            } 
+       if (c->active)
+         active_day = day;            
     }
 
   gtk_widget_draw (draw, NULL);
@@ -394,10 +426,10 @@ month_view_update ()
 }
 
 static void
-changed_callback(GtkWidget *widget,
-		 gpointer d)
+changed_callback(GtkWidget *widget, gpointer d)
 {
   viewtime = gtk_date_sel_get_time (GTK_DATE_SEL (widget));
+  
   month_view_update ();
 }
 
@@ -427,15 +459,170 @@ resize_table (GtkWidget *widget,
       ys = height / 7;
 
       if (ys > xs)
-	ys = xs;
+        ys = xs;
 
       gtk_widget_draw (draw, NULL);
     }
 }
 
+static gboolean
+month_view_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkWidget *user_data)
+{
+  struct render_ctl *c;
+ 
+  if (k->keyval == GDK_Escape)
+    {
+      c = &rc[active_day];
+      if (c->valid)
+        {
+          if (pop_window) 
+            gtk_widget_destroy (pop_window);
+            pop_window = NULL;
+            c_old = NULL;
+        }    
+    }
+    
+  if (k->keyval == GDK_Right) 
+    {
+      if ((active_day < TOTAL_DAYS) && (rc[active_day+1].valid))
+        {
+          active_day++;
+          viewtime = time_from_day(rc[active_day].popup.year,
+                                   rc[active_day].popup.month,
+                                   rc[active_day].popup.day);
+          month_view_update();
+        }
+      else
+        gtk_widget_child_focus(gtk_widget_get_toplevel(GTK_WIDGET(widget)),
+                             GTK_DIR_DOWN);  
+      return TRUE;
+    }
+    
+  if (k->keyval == GDK_Left) 
+    {
+      if ((active_day) && (rc[active_day-1].valid))
+        {
+          active_day--;
+          viewtime = time_from_day(rc[active_day].popup.year,
+                                   rc[active_day].popup.month,
+                                   rc[active_day].popup.day);
+          month_view_update();
+        }
+      else
+        gtk_widget_child_focus(gtk_widget_get_toplevel(GTK_WIDGET(widget)),
+                             GTK_DIR_UP);  
+      return TRUE;
+    }
+
+  if (k->keyval == GDK_Down) 
+    {
+      if ((active_day < TOTAL_DAYS-7) && (rc[active_day+7].valid))
+        {
+          active_day+=7;
+          viewtime = time_from_day(rc[active_day].popup.year,
+                                   rc[active_day].popup.month,
+                                   rc[active_day].popup.day);
+          month_view_update();
+        }
+      else
+        gtk_widget_child_focus(gtk_widget_get_toplevel(GTK_WIDGET(widget)),
+                             GTK_DIR_DOWN);  
+      return TRUE;
+    }
+    
+  if (k->keyval == GDK_Up) 
+    {
+      if ((active_day >= 7) && (rc[active_day-7].valid))
+        {
+          active_day -= 7;
+          viewtime = time_from_day(rc[active_day].popup.year,
+                                   rc[active_day].popup.month,
+                                   rc[active_day].popup.day);
+          month_view_update();
+        }
+      else
+        gtk_widget_child_focus(gtk_widget_get_toplevel(GTK_WIDGET(widget)),
+                             GTK_DIR_UP);  
+      return TRUE;
+    }
+    
+  if (k->keyval == GDK_space)
+    {
+      c = &rc[active_day];
+      if (c->valid)
+        {
+          if (pop_window) 
+            gtk_widget_destroy (pop_window);
+              if (c != c_old) 
+            {
+              pop_window = day_popup (main_window, &c->popup, TRUE);
+              c_old = c;
+            }
+           else 
+            {
+              pop_window = NULL;
+              c_old = NULL;
+            }
+         }
+         return TRUE;
+     }  
+     
+  if (k->keyval == GDK_Return)
+    {
+      c = &rc[active_day];
+      if (c->valid)
+        {
+          struct tm tm;
+          time_t selected_time;
+          localtime_r (&viewtime, &tm);
+          tm.tm_year = c->popup.year;//- 1900;
+          tm.tm_mon = c->popup.month;
+          tm.tm_mday = c->popup.day;
+          tm.tm_hour = 0;
+          tm.tm_min = 0;
+          tm.tm_sec = 0;
+          selected_time = mktime (&tm);
+          if (pop_window) 
+              gtk_widget_destroy (pop_window);
+          set_time_and_day_view (selected_time);    
+        }
+      return TRUE; 
+    }
+  
+  return FALSE;
+}
+
+void
+month_free_lists()
+{
+  guint day;
+  struct tm tm_start;
+  guint days;
+  guint year, month;
+  struct tm today;
+  time_t t;
+
+  time (&t);
+  localtime_r (&t, &today);
+
+  localtime_r (&viewtime, &tm_start);
+  year = tm_start.tm_year + 1900;
+  month = tm_start.tm_mon;
+
+  days = days_in_month (year, month);
+
+  for (day = 1; day <= days; day++)
+    if (day_events[day]) 
+      {
+        event_db_list_destroy (day_events[day]);
+        day_events[day] = NULL;
+      }
+}
+
 GtkWidget *
 month_view(void)
 {
+  int day;
   GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
 
   draw = gtk_drawing_area_new ();
@@ -445,6 +632,7 @@ month_view(void)
 
   datesel = gtk_date_sel_new (GTKDATESEL_MONTH);
   gtk_widget_show (datesel);
+  gtk_widget_grab_focus(datesel);
 
   gtk_box_pack_start (GTK_BOX (vbox), datesel, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), draw, TRUE, TRUE, 0);
@@ -455,13 +643,25 @@ month_view(void)
   g_signal_connect(G_OBJECT (draw), "button-press-event",
                    G_CALLBACK (button_press), NULL);
 
-  gtk_widget_add_events (GTK_WIDGET (draw), GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
+  gtk_widget_add_events (GTK_WIDGET (draw), 
+            GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 
+  g_signal_connect (G_OBJECT (datesel), "key_press_event", 
+		    G_CALLBACK (month_view_key_press_event), NULL);
+            
+  gtk_widget_add_events (GTK_WIDGET (datesel), 
+            GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
+            
   g_signal_connect(G_OBJECT (datesel), "changed",
                      G_CALLBACK (changed_callback), NULL);
 
   g_object_set_data (G_OBJECT (vbox), "update_hook",
                      (gpointer) update_hook_callback);
+                     
+  for (day = 0; day < TOTAL_DAYS; day++)
+      rc[day].initialized = FALSE;
+  
+  g_object_set_data(G_OBJECT(main_window),"datesel-month",datesel);
 
   return vbox;
 }
